<?php
namespace Ep\Gath;

use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use Throwable;
use Zend\Log\LoggerInterface;
use Zend\Stdlib\ArrayUtils;

/**
 * Class Gath
 *
 * @package Ep\Gath
 * @author Elektronika Perhoč <support@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class Gath
{
    const SORT_DATE_DESC  = 1;
    const SORT_PRICE_ASC  = 2;
    const SORT_PRICE_DESC = 3;
    const SORT_STOCK_DESC = 4;
    const SORT_NAME_DESC  = 5;
    const SORT_NAME_ASC   = 6;

    const ID_OFFSET = 1e7;
    const ID_OFFSET_SIMPLE = self::ID_OFFSET;
    const ID_OFFSET_GROUPED = self::ID_OFFSET_SIMPLE + 1e7;
    const ID_OFFSET_VARIABLE = self::ID_OFFSET_GROUPED + 1e7;
    const ID_OFFSET_VARIATION = self::ID_OFFSET_VARIABLE + 1e7;
    const ID_OFFSET_TIRES = self::ID_OFFSET_VARIATION + 1e7;

    /** @var array */
    protected $config;
    /** @var ClientInterface|Client */
    protected $http;
    /** @var LoggerInterface */
    protected $logger;

    /**
     * Gath constructor.
     *
     * @param array $config
     * @param ClientInterface $http
     * @param LoggerInterface $logger
     */
    public function __construct(array $config, ClientInterface $http, LoggerInterface $logger)
    {
        $this->config = $config;
        $this->http   = $http;
        $this->logger = $logger;
    }

    /**
     * Da li smo u debug modu?
     *
     * @return bool
     */
    public function isDebug(): bool
    {
        return $this->config['debug'] ?? false;
    }

    /**
     * Da li je ovo veleprodajni shop?
     *
     * @return bool
     */
    public function isWholesale(): bool
    {
        return $this->config['wholesale'] ?? false;
    }

    /**
     * Vraća osnovni uri servisa sa slikama.
     *
     * @return string
     */
    public function getImageBaseUri(): string
    {
        return $this->config['image_base_uri'] ?? '';
    }

    /**
     * Da li prikazujemo kratki opis.
     *
     * @return bool
     */
    public function getShowShortDescription(): bool
    {
        return $this->config['show_short_description'] ?? true;
    }

    /**
     * @param int $categoryId
     *
     * @return array
     */
    public function getBrandsByCategory(int $categoryId): array
    {
        return $this->request('POST', 'product/brands', [
            'CategoryId' => $categoryId,
        ]);
    }

    public function getBrands():  array
    {
        return $this->request('GET', 'product/brands');
    }

    public function getCategories($parent, int $depth): array
    {
        $params = ['Depth' => $depth];

        if ($parent) {
            $params['Parent'] = $parent;
        }

        return $this->request('POST', 'category', $params);
    }

    /**
     * Vraća istaknute proizvode.
     *
     * @param int $orderBy
     * @param int $limit
     * @param int $offset
     *
     * @return array
     */
    public function getFeaturedProducts(int $orderBy = 1, int $limit = 10, int $offset = 0): array
    {
        return $this->request('POST', 'products/featured', [
            'OrderBy' => $orderBy,
            'Limit' => $limit,
            'Offset' => $offset,
        ]);
    }

    /**
     * Vraća proizvođače motocikala.
     *
     * @return array
     */
    public function getMotorcycleBrands(): array
    {
        return $this->request('GET', 'moto/brands');
    }

    /**
     * Vraća modele motocikala za datog proizvođača.
     *
     * @param int $motorcycleBrandId
     *
     * @return array
     */
    public function getMotorcycleModels(int $motorcycleBrandId): array
    {
        return $this->request('POST', 'moto/models', [
            'Brands' => [$motorcycleBrandId],
        ]);
    }

    /**
     * @param int $motorcycleId
     *
     * @return array|null
     */
    public function getMotorcycle(int $motorcycleId): ?array
    {
        $motorcycles = $this->request('GET', 'moto/motorcycles', [
            'Id' => $motorcycleId,
        ]);

        return count($motorcycles['Results']) > 0 ? $motorcycles['Results'][0] : null;
    }

    /**
     * Vraća motocikle za dati model.
     *
     * @param int $modelId
     *
     * @return array
     */
    public function getMotorcyclesByModel(int $modelId): array
    {
        return $this->request('POST', 'moto/motorcycles', [
            'Models' => [$modelId],
        ]);
    }

    /**
     * Vraća cijene po šiframa proizvoda.
     *
     * @param array $skus
     * @param array $tireIds
     * @return array
     */
    public function getPricesBySkus(array $skus, array $tireIds = []): array
    {
        $user = wp_get_current_user();
        $username = $user->ID > 0 ? $user->user_login : null;

        $params = [
            'Username' => $username,
            'PriceRabatePolicy' => $this->config['wholesale'] ?? false,
            'Discount' => true,
            'Codes' => $skus
        ];

        if (!empty($tireIds)) {
            $params['TireIds'] = $tireIds;
        }

        return $this->request('POST', 'prices/by-codes', $params);
    }

    /**
     * Vraća proizvode po wooIdu kategorije.
     *
     * @param int $categoryId
     * @param int $sort
     * @param int $limit
     * @param int $offset
     *
     * @param array|null $brands
     * @return array
     */
    public function getProductsByCategory(
        int $categoryId,
        int $sort = self::SORT_DATE_DESC,
        int $limit = 10,
        int $offset = 0,
        array $brands = null
    ): array {
        $params = [
            'CategoryId'      => $categoryId,
            'OrderBy'         => $sort,
            'Limit'           => $limit,
            'Offset'          => $offset,
            'SortNoStockLast' => $this->config['sort_no_stock_last'] ?? false,
        ];

        if (null !== $brands && !empty($brands)) {
            $params['BrandIds'] = array_values($brands);
        }

        return $this->request('POST', 'product/bycategory', $params);
    }

    public function getTires(
        array $filters,
        int $order = self::SORT_DATE_DESC, int $limit = 10, int $offset = 0
    )
    {
        $params = [
            'Limit' => $limit,
            'Offset' => $offset,
            'OrderBy' => $order,
            'SortNoStockLast' => $this->config['sort_no_stock_last'] ?? false,
        ];

        if (!empty($filters['width'])) $params['TireWidth'] = (int)$filters['width'];
        if (!empty($filters['height'])) $params['TireHeight'] = (int)$filters['height'];
        if (!empty($filters['diameter'])) $params['TireDiameter'] = (float)$filters['diameter'];
        if (!empty($filters['season'])) $params['Season'] = [ $filters['season'] ];
        if (!empty($filters['type'])) $params['VehicleType'] = [ $filters['type'] ];
        if (!empty($filters['brand'])) $params['BrandName'] = [ $filters['brand'] ];
        if (!empty($filters['loadIndex'])) $params['LoadIndex1'] = [ (int)$filters['loadIndex'] ];
        if (!empty($filters['speedIndex'])) $params['SpeedIndex'] = [ $filters['speedIndex'] ];

//        var_dump($params);
        return $this->request('POST', 'product/tyres', $params);
    }

    /**
     * Vraća proizvode po id-u.
     *
     * @param array $productIds
     *
     * @param int $limit
     * @param int $offset
     * @return array
     */
    public function getProductsByIds(array $productIds, int $limit = 1000, int $offset = 0): array
    {
        $dataLinks = [
            'TireIds'     => [],
            'SimpleIds'   => [],
            'GroupedIds'  => [],
            'VariableIds' => [],
            'VariationIds' => [],
            'Limit'       => $limit,
            'Offset'      => $offset,
        ];

        // pretvaramo id-eve u DataLinkove i razdvajamo po vrsti proizvoda.
        foreach ($productIds as $productId) {
            if ($productId > static::ID_OFFSET_TIRES) {
                $dataLinks['TireIds'][] = intval($productId - static::ID_OFFSET_TIRES);
            } else if ($productId > static::ID_OFFSET_VARIATION) {
                $dataLinks['VariationIds'][] = intval($productId - static::ID_OFFSET_VARIATION);
            } else if ($productId > static::ID_OFFSET_VARIABLE) {
                $dataLinks['VariableIds'][] = intval($productId - static::ID_OFFSET_VARIABLE);
            } else if ($productId > static::ID_OFFSET_GROUPED) {
                $dataLinks['GroupedIds'][] = intval($productId - static::ID_OFFSET_GROUPED);
            } else {
                $dataLinks['SimpleIds'][] = intval($productId - static::ID_OFFSET_SIMPLE);
            }
        }

        $result = [];

        foreach (['TireIds', 'GroupedIds', 'SimpleIds', 'VariationIds', 'VariableIds'] as $type) {
            if (count($dataLinks[$type]) === 0) {
                continue;
            }

            $result = ArrayUtils::merge($result, $this->request('POST', 'product/byids', [
                $type => $dataLinks[$type],
                'Limit' => $dataLinks['Limit'],
                'Offset' => $dataLinks['Offset'],
            ]));
        }

        // Ovo ne valja nikako
        $charConvert = [
            'Ĺľ' => 'ž',
            '&scaron;' => 'š',
            'ÄŤ' => 'č',
            'Ä‡' => 'ć',
            'Ä‘' => 'đ',
            'Ä†' => 'Č',
            'ÄŚ' => 'Ć',
            'Ĺ˝' => 'Ž'
        ];
        foreach ($result['Results'] as &$res) {
            if (isset($res['description'])) {
                $res['description'] = str_replace(array_keys($charConvert), array_values($charConvert), $res['description']);
//                $res['description'] = strip_tags($res['description'], '<span>');

            }

        }

        return $result;
    }

    /**
     * Vraća simple proizvode po šiframa.
     *
     * @param array $skus
     *
     * @return array
     */
    public function getProductsByskus(array $skus): array
    {
        return $this->request('POST', 'product/byids', [
            'SimpleCodes' => $skus,
            'Limit' => 1000,
        ]);
    }

    /**
     * @param array $tecDocPairs
     * @param int $order
     * @param int $limit
     * @param int $offset
     *
     * @return mixed
     */
    public function getProductsByTecDocPairs(array $tecDocPairs, int $order, int $limit = 10, int $offset = 0): array
    {
        $params = [
            'TecDocKeys'      => $tecDocPairs,
            'OrderBy'         => $order,
            'Limit'           => $limit,
            'Offset'          => $offset,
            'SortNoStockLast' => $this->config['sort_no_stock_last'] ?? false,
        ];

        return $this->request('POST', 'product/bytecdocids', $params);
    }

    /**
     * @param $motorcycleId
     * @param $genericArticleIds
     * @param int $orderBy
     * @param int $limit
     * @param int $offset
     *
     * @return array
     */
    public function getProductsByMotorcycle(
        int $motorcycleId,
        array $genericArticleIds,
        int $orderBy,
        int $limit = 10,
        int $offset = 0
    ): array {
        return $this->request('POST', 'moto/articles', [
            'MotoId' => $motorcycleId,
            'GenArtIds' => $genericArticleIds,
            'OrderBy' => $orderBy,
            'Limit' => $limit,
            'Offset' => $offset,
            'SortNoStockLast' => $this->config['sort_no_stock_last'] ?? false,
        ]);
    }

    /**
     * Vraća stanje zalihe po šifri proizvoda.
     *
     * @param array $skus
     *
     * @param array $tireIds
     * @return array
     */
    public function getStockBySkus(array $skus, array $tireIds = []): array
    {
        $params = [
            'Codes' => $skus
        ];

        if (!empty($tireIds)) {
            $params['TireIds'] = $tireIds;
        }

        return $this->request('POST', 'stock/by-codes', $params);
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Upućuje zahtjev prema servisu.
     *
     * @param $method
     * @param $path
     * @param array $params
     *
     * @return array
     */
    protected function request(string $method = 'GET', string $path = '', array $params = []): array
    {
        try {
            if ('POST' === $method) {
                $request = [
                    'json' => $params,
                    'headers' => [
                        'Content-Type' => 'application/json',
                    ],
                ];
            } else {
                $request = [
                    'query' => $params,
                ];
            }

            try {
                if ($this->isDebug()) {
                    ob_start();
                    /** @noinspection PhpUnhandledExceptionInspection */
                    $response = $this->http->request($method, $path, $request);
                    $debug    = ob_get_clean();
                    $this->logger->debug($debug);
                } else {
                    /** @noinspection PhpUnhandledExceptionInspection */
                    $response = $this->http->request($method, $path, $request);
                }
            } catch (Throwable $e) {
                $this->logger->crit($e->getMessage() . "\n" . json_encode($params));
                throw new \RuntimeException($e->getMessage());
            }

            if (300 <= $response->getStatusCode()) {
                $this->logger->err(sprintf('Došlo je do greške z Gathom za: %s %s', $path, json_encode($params)));
                throw new \RuntimeException('Gath greška: ' . $response->getReasonPhrase());
            }

            $jsonContent = $response->getBody()->getContents();
            $data = json_decode($jsonContent, true);

            if (null === $data) {
                $this->logger->err(sprintf(
                    "Greška prilikom dekodiranja JSON-a: %s.\nJSON: %s",
                    json_last_error_msg(),
                    $jsonContent
                ));
                throw new \RuntimeException('Greška prilikom dekodiranja JSON-a.');
            }

            if (!isset($data['Results'])) {
                throw new \RuntimeException("Objekt iz Gatha ne sadrži polje 'Results'.");
            }

            if (!is_array($data['Results'])) {
                throw new \RuntimeException(sprintf(
                    "Polje 'Results' mora sadržavati array, a ne %s",
                    is_object($data) ? get_class($data['Results']) : gettype($data['Results'])
                ));
            }
        } catch (Throwable $e) {
            $data = [
                'Results' => [],
                'RecordCount' => 0,
            ];
        }
        
        return $data;
    }
}
